@@ -2,7 +2,7 @@ use strict;
 use warnings;
 package Log::Dispatchouli;
 # ABSTRACT: a simple wrapper around Log::Dispatch
-$Log::Dispatchouli::VERSION = '2.015';
+$Log::Dispatchouli::VERSION = '2.022';
 use Carp ();
 use File::Spec ();
 use Log::Dispatch;
@@ -123,6 +123,7 @@ our @CARP_NOT = qw(Log::Dispatchouli::Proxy);
 #pod                 fatal log messages will not be logged to these
 #pod                 (default: stderr)
 #pod   config_id   - a name for this logger's config; rarely needed!
+#pod   syslog_socket - a value for Sys::Syslog's "socket" arg; default: "native"
 #pod
 #pod The log path is either F</tmp> or the value of the F<DISPATCHOULI_PATH> env var.
 #pod
@@ -146,19 +147,11 @@ sub new {
         : ('stderr');
   };
 
-  my $pid_prefix = exists $arg->{log_pid} ? $arg->{log_pid} : 1;
-
-  my $self = bless {} => $class;
-
-  my $log = Log::Dispatch->new(
-    $pid_prefix
-    ? (
-        callbacks => sub {
-	  "[$$] ". {@_}->{message}
-	},
-      )
-    : ()
-  );
+  my $log = Log::Dispatch->new;
+  my $self = bless {
+    dispatcher => $log,
+    log_pid    => (exists $arg->{log_pid} ? $arg->{log_pid} : 1),
+  } => $class;
 
   if ($arg->{to_file}) {
     require Log::Dispatch::File;
@@ -182,11 +175,19 @@ sub new {
         mode      => 'append',
         callbacks => do {
           if (my $format = $arg->{file_format}) {
-            sub { $format->({@_}->{message}) }
+            sub {
+              my $message = {@_}->{message};
+              $message = "[$$] $message" if $self->{log_pid};
+              $format->($message)
+            };
           } else {
             # The time format returned here is subject to change. -- rjbs,
             # 2008-11-21
-            sub { (localtime) . ' ' . {@_}->{message} . "\n" }
+            sub {
+              my $message = {@_}->{message};
+              $message = "[$$] $message" if $self->{log_pid};
+              (localtime) . " $message\n";
+            };
           }
         },
       )
@@ -194,20 +195,10 @@ sub new {
   }
 
   if ($arg->{facility} and not $self->env_value('NOSYSLOG')) {
-    require Log::Dispatch::Syslog;
-    $log->add(
-      Log::Dispatch::Syslog->new(
-        name      => 'syslog',
-        min_level => 'debug',
-        facility  => $arg->{facility},
-        ident     => $ident,
-        logopt    => 'pid',
-        socket    => 'native',
-        callbacks => sub {
-          ( my $m = {@_}->{message} ) =~ s/\n/<LF>/g;
-          $m
-        },
-      ),
+    $self->setup_syslog_output(
+      facility  => $arg->{facility},
+      socket    => $arg->{syslog_socket},
+      ident     => $ident,
     );
   }
 
@@ -219,38 +210,77 @@ sub new {
         name      => 'self',
         min_level => 'debug',
         array     => $self->{events},
+        ($self->{log_pid} ? (callbacks => sub { "[$$] ". {@_}->{message} })
+                          : ())
       ),
     );
   }
 
-  DEST: for my $dest (qw(err out)) {
-    next DEST unless $arg->{"to_std$dest"};
-    require Log::Dispatch::Screen;
-    $log->add(
-      Log::Dispatch::Screen->new(
-        name      => "std$dest",
-        min_level => 'debug',
-        stderr    => ($dest eq 'err' ? 1 : 0),
-        callbacks => sub { +{@_}->{message} . "\n" },
-        ($quiet_fatal{"std$dest"} ? (max_level => 'info') : ()),
-      ),
-    );
-  }
-
-  $self->{dispatcher} = $log;
   $self->{prefix}     = $arg->{prefix};
   $self->{ident}      = $ident;
   $self->{config_id}  = $config_id;
 
+  DEST: for my $dest (qw(err out)) {
+    next DEST unless $arg->{"to_std$dest"};
+    my $method = "enable_std$dest";
+
+    $self->$method;
+  }
+
   $self->{debug}  = exists $arg->{debug}
                   ? ($arg->{debug} ? 1 : 0)
                   : ($self->env_value('DEBUG') ? 1 : 0);
+  $self->{muted}  = $arg->{muted};
 
-  $self->{fail_fatal} = exists $arg->{fail_fatal} ? $arg->{fail_fatal} : 1;
+  $self->{quiet_fatal} = \%quiet_fatal;
+  $self->{fail_fatal}  = exists $arg->{fail_fatal} ? $arg->{fail_fatal} : 1;
 
   return $self;
 }
 
+for my $dest (qw(out err)) {
+  my $name = "std$dest";
+  my $code = sub {
+    return if $_[0]->dispatcher->output($name);
+
+    my $callback = $_[0]->{log_pid} ? sub { "[$$] " . ({@_}->{message}) . "\n" }
+                                    : sub {           ({@_}->{message}) . "\n" };
+
+    $_[0]->dispatcher->add(
+      $_[0]->stdio_dispatcher_class->new(
+        name      => "std$dest",
+        min_level => 'debug',
+        stderr    => ($dest eq 'err' ? 1 : 0),
+        callbacks => $callback,
+        ($_[0]{quiet_fatal}{"std$dest"} ? (max_level => 'info') : ()),
+      ),
+    );
+  };
+
+  no strict 'refs';
+  *{"enable_std$dest"} = $code;
+}
+
+sub setup_syslog_output {
+  my ($self, %arg) = @_;
+
+  require Log::Dispatch::Syslog;
+  $self->{dispatcher}->add(
+    Log::Dispatch::Syslog->new(
+      name      => 'syslog',
+      min_level => 'debug',
+      facility  => $arg{facility},
+      ident     => $arg{ident},
+      logopt    => ($self->{log_pid} ? 'pid' : ''),
+      socket    => $arg{socket} || 'native',
+      callbacks => sub {
+        ( my $m = {@_}->{message} ) =~ s/\n/<LF>/g;
+        $m
+      },
+    ),
+  );
+}
+
 #pod =method log
 #pod
 #pod   $logger->log(@messages);
@@ -629,6 +659,19 @@ sub logger { $_[0] }
 
 sub dispatcher   { $_[0]->{dispatcher} }
 
+#pod =method stdio_dispatcher_class
+#pod
+#pod This method is an experimental feature to allow you to pick an alternate
+#pod dispatch class for stderr and stdio.  By default, Log::Dispatch::Screen is
+#pod used.  B<This feature may go away at any time.>
+#pod
+#pod =cut
+
+sub stdio_dispatcher_class {
+  require Log::Dispatch::Screen;
+  return 'Log::Dispatch::Screen';
+}
+
 #pod =head1 METHODS FOR API COMPATIBILITY
 #pod
 #pod To provide compatibility with some other loggers, most specifically
@@ -696,7 +739,7 @@ Log::Dispatchouli - a simple wrapper around Log::Dispatch
 
 =head1 VERSION
 
-version 2.015
+version 2.022
 
 =head1 SYNOPSIS
 
@@ -765,6 +808,7 @@ Valid arguments are:
                 fatal log messages will not be logged to these
                 (default: stderr)
   config_id   - a name for this logger's config; rarely needed!
+  syslog_socket - a value for Sys::Syslog's "socket" arg; default: "native"
 
 The log path is either F</tmp> or the value of the F<DISPATCHOULI_PATH> env var.
 
@@ -870,6 +914,12 @@ current logger will not throw an exception, and will simply do no thing.
 This returns the underlying Log::Dispatch object.  This is not the method
 you're looking for.  Move along.
 
+=head2 stdio_dispatcher_class
+
+This method is an experimental feature to allow you to pick an alternate
+dispatch class for stderr and stdio.  By default, Log::Dispatch::Screen is
+used.  B<This feature may go away at any time.>
+
 =head1 LOGGER PREFIX
 
 Log messages may be prepended with information to set context.  This can be set
@@ -1044,7 +1094,7 @@ Ricardo SIGNES <rjbs@cpan.org>
 
 =head1 CONTRIBUTORS
 
-=for stopwords Christopher J. Madsen Dagfinn Ilmari Mannsåker George Hartzell Matt Phillips Olivier Mengué Randy Stauner Sawyer X
+=for stopwords Christopher J. Madsen Dagfinn Ilmari Mannsåker Dan Book George Hartzell Jon Stuart Matt Phillips Olivier Mengué Randy Stauner Ricardo Signes Sawyer X
 
 =over 4
 
@@ -1058,10 +1108,18 @@ Dagfinn Ilmari Mannsåker <ilmari@ilmari.org>
 
 =item *
 
+Dan Book <grinnz@gmail.com>
+
+=item *
+
 George Hartzell <hartzell@alerce.com>
 
 =item *
 
+Jon Stuart <jon@fastmailteam.com>
+
+=item *
+
 Matt Phillips <mattp@cpan.org>
 
 =item *
@@ -1074,13 +1132,21 @@ Randy Stauner <randy@magnificent-tears.com>
 
 =item *
 
+Ricardo Signes <rjbs@semiotic.systems>
+
+=item *
+
+Ricardo Signes <rjbs@users.noreply.github.com>
+
+=item *
+
 Sawyer X <xsawyerx@cpan.org>
 
 =back
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2016 by Ricardo SIGNES.
+This software is copyright (c) 2020 by Ricardo SIGNES.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
